/*
 * gMapMaker
 * Copyright (C) 2007  Damien Debin
 * http://debin.net/gMapMaker/
 * 
 * This file is part of gMapMaker.
 * 
 * Foobar is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Foobar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Globalization;
using System.Diagnostics;
using System.Threading;
using System.Text.RegularExpressions;

namespace gMapMaker
{
  public partial class MainForm : Form
  {
    public static CultureInfo ciUS;

    ComponentResourceManager resources;
    ProgressWindow pw;

    public MainForm()
    {
      ciUS = CultureInfo.GetCultureInfo("en-US");

      InitializeComponent();

      resources = new ComponentResourceManager(typeof(MainForm));

      InitUIwithValues();
    }

    private void InitUIwithValues()
    {
      if (!Path.IsPathRooted(Properties.Settings.Default.CacheFullPath))
      {
        Properties.Settings.Default.CacheFullPath = Path.GetFullPath(Properties.Settings.Default.CacheFullPath);
      }

      tlLatTextBox.Text = Properties.Settings.Default.TopLat.ToString(ciUS.NumberFormat);
      tlLongTextBox.Text = Properties.Settings.Default.LeftLong.ToString(ciUS.NumberFormat);
      brLatTextBox.Text = Properties.Settings.Default.BottomLat.ToString(ciUS.NumberFormat);
      brLongTextBox.Text = Properties.Settings.Default.RightLong.ToString(ciUS.NumberFormat);
      googleMVtextBox.Text = Properties.Settings.Default.CacheFullPath;
      zoomLevelComboBox.SelectedItem = Properties.Settings.Default.ZoomLevel.ToString();
      grayscaleCheckBox.Checked = Properties.Settings.Default.GrayScale;
      imageFormatComboBox.Text = Properties.Settings.Default.ImageFormat;
      tileTypeComboBox.SelectedIndex = Properties.Settings.Default.TileType;
      ditherCheckBox.Checked = Properties.Settings.Default.Dither;
      proxyUseCheckBox.Checked = Properties.Settings.Default.UseProxyForSatelliteDownload;
      waitRequestsCheckBox.Checked = Properties.Settings.Default.SlowDown;
      delayNumericUpDown.Value = Properties.Settings.Default.SlowDownDelay;
      proxyListUrlsTextBox.Text = Properties.Settings.Default.ProxyListURLs;
      slicingComboBox.SelectedIndex = (int)Math.Sqrt(Properties.Settings.Default.NumberOfSlices) - 1;
      operatingModeComboBox.SelectedIndex = Properties.Settings.Default.OperatingMode;
      proxyListRegexpTextBox.Text = Properties.Settings.Default.ProxyListRegexp;

      //operatingModeComboBox.Items.RemoveAt(4);

      uint transp = Math.Max(Properties.Settings.Default.DrivingMapTransparency, 0);
      transp = Math.Min(transp, 100);
      transpComboBox.SelectedItem = transp.ToString() + '%';

      errorProvider.SetError(tlLongTextBox, "");
      errorProvider.SetError(tlLatTextBox, "");
      errorProvider.SetError(brLongTextBox, "");
      errorProvider.SetError(brLatTextBox, "");

      ValidateChildren(ValidationConstraints.Enabled);

      updateStatus(null, null);
    }

    private static void ShowErrorBox(string message)
    {
      MessageBox.Show(Properties.Resources.ApplicationErrorText + "\n\n" + message, Properties.Resources.ApplicationErrorTitle, MessageBoxButtons.OK, MessageBoxIcon.Warning, MessageBoxDefaultButton.Button1, MessageBoxOptions.ServiceNotification);
    }

    #region ComputeMapBackgroundWorker

    class WorkerArguments
    {
      public GMapTile TopLeft;
      public GMapTile BottomRight;
      public string Filename;
    }

    private void computeMapBackgroundWorker_DoWork(object sender, DoWorkEventArgs e)
    {
#if DEBUG
      TextWriterTraceListener textWriterTraceListener = new TextWriterTraceListener("log.txt");
      textWriterTraceListener.TraceOutputOptions = TraceOptions.DateTime;
      Debug.Listeners.Add(textWriterTraceListener);
#endif

      // Get the BackgroundWorker that raised this event.
      BackgroundWorker worker = sender as BackgroundWorker;

      WorkerArguments arg = (WorkerArguments)e.Argument;

      if (arg.TopLeft.Zoom != arg.BottomRight.Zoom)
      {
        throw new ArgumentException();
      }

      //check if cache folder exists
      if (!Directory.Exists(Properties.Settings.Default.CacheFullPath))
      {
        Directory.CreateDirectory(Properties.Settings.Default.CacheFullPath);
      }

      BigImage bi = new BigImage(arg.TopLeft.Quadtree, arg.BottomRight.Quadtree, ReportProgressFunction, worker);
      bi.grayscale = Properties.Settings.Default.GrayScale;
      bi.operatingMode = Properties.Settings.Default.TileType;
      bi.dither8bpp = Properties.Settings.Default.Dither;

      if ((Properties.Settings.Default.OperatingMode == 0) || (Properties.Settings.Default.OperatingMode == 3) || (Properties.Settings.Default.OperatingMode == 4))
      {
        bi.imageFormat = FreeImageAPI.FREE_IMAGE_FORMAT.FIF_UNKNOWN;
      }
      else
      {
        switch (Properties.Settings.Default.ImageFormat)
        {
          case "PNG":
            bi.imageFormat = FreeImageAPI.FREE_IMAGE_FORMAT.FIF_PNG;
            bi.saveflags = FreeImageAPI.LoadSaveFlags.PNG_DEFAULT;
            break;
          case "JPEG":
            bi.imageFormat = FreeImageAPI.FREE_IMAGE_FORMAT.FIF_JPEG;
            bi.saveflags = FreeImageAPI.LoadSaveFlags.JPEG_QUALITYSUPERB;
            break;
          case "TIFF":
            bi.imageFormat = FreeImageAPI.FREE_IMAGE_FORMAT.FIF_TIFF;
            bi.saveflags = FreeImageAPI.LoadSaveFlags.TIFF_PACKBITS;
            break;
        }
      }

      bi.BuildBigImage(arg.Filename);

      if (worker.CancellationPending)
      {
        return;
      }

      if (Properties.Settings.Default.OperatingMode == 2)
      {
        ReportProgressFunction("Saving map", 0);

        OziExplorerMap oem = new OziExplorerMap();
        oem.MapName = Path.GetFileName(arg.Filename);
        oem.ImageFileFullPath = arg.Filename;
        oem.ImageHeight = bi.ImageHeight;
        oem.ImageWidth = bi.ImageWidth;
        oem.OnePixelLength = arg.TopLeft.OnePixelLength;
        oem.ReferencePoints.Add(new MapReferencePoint(0, 0, arg.TopLeft.TopLat, arg.TopLeft.LeftLong));
        oem.ReferencePoints.Add(new MapReferencePoint(bi.ImageWidth, bi.ImageHeight, arg.BottomRight.BottomLat, arg.BottomRight.RightLong));
        oem.ReferencePoints.Add(new MapReferencePoint(0, bi.ImageHeight, arg.BottomRight.BottomLat, arg.TopLeft.LeftLong));
        oem.ReferencePoints.Add(new MapReferencePoint(bi.ImageWidth, 0, arg.TopLeft.TopLat, arg.BottomRight.RightLong));

        int YY = arg.BottomRight.Y - arg.TopLeft.Y;
        int XX = arg.BottomRight.X - arg.TopLeft.X;

        if ((XX > 1) && (YY > 1))
        {
          double R = Math.Abs(((double)XX + 1.0) / ((double)YY + 1.0));
          int NY = (int)Math.Sqrt(30.0 / R);
          int NX = 30 / NY;
          double stepY = (double)YY / Math.Min(YY, NY - 1);
          double stepX = (double)XX / Math.Min(XX, NX - 1);

          int y = 0;
          double dy = 0;
          for (; y <= YY; dy += stepY, y = (int)Math.Round(dy))
          {
            int x = 0;
            double dx = 0;
            for (; x <= XX; dx += stepX, x = (int)Math.Round(dx))
            {
              if (((x == 0) && (y == 0)) || ((x == 0) && (y == YY)) || ((x == XX) && (y == 0)) || ((x == XX) && (y == YY)))
              {
                continue;
              }
              GMapTile g = new GMapTile(arg.TopLeft.X + x, arg.TopLeft.Y + y, arg.TopLeft.Zoom);
              oem.ReferencePoints.Add(new MapReferencePoint(x * GMapTile.TILE_SIZE, y * GMapTile.TILE_SIZE, g.TopLat, g.LeftLong));
            }
          }
        }

        oem.SaveToMap(Path.ChangeExtension(arg.Filename, ".map"));
      }

      ReportProgressFunction("Map saved", 20.0);
    }

    struct ReportInfo
    {
      public double inc;
      public string status;
      public ReportInfo(double inc_, string status_)
      {
        inc = inc_;
        status = status_;
      }
    }

    private void ReportProgressFunction(string message, double inc)
    {
      computeMapBackgroundWorker.ReportProgress(0, new ReportInfo(inc / 220.0, message));
    }

    private void computeMapBackgroundWorker_ProgressChanged(object sender, ProgressChangedEventArgs e)
    {
      ReportInfo r = (ReportInfo)e.UserState;
      pw.SetProgressValue(r.inc);
      pw.SetStatusLabel(r.status);
    }

    private void computeMapBackgroundWorker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
    {
      // First, handle the case where an exception was thrown.
      if ((e.Error != null) && !e.Cancelled)
      {
        Cursor.Current = Cursors.Default;
        ShowErrorBox(e.Error.Message);
      }
      else if (e.Cancelled)
      {
        // Next, handle the case where the user canceled 
        // the operation.
        // Note that due to a race condition in 
        // the DoWork event handler, the Cancelled
        // flag may not have been set, even though
        // CancelAsync was called.
      }
      pw.closingAsked = true;
      pw.Close();
    }

    #endregion

    private void chooseFolderButton_Click(object sender, EventArgs e)
    {
      folderBrowserDialog.ShowDialog();
      if (!String.IsNullOrEmpty(folderBrowserDialog.SelectedPath))
      {
        googleMVtextBox.Text = folderBrowserDialog.SelectedPath;
        Properties.Settings.Default.CacheFullPath = googleMVtextBox.Text;
      }
    }

    private void saveButton_Click(object sender, EventArgs e)
    {
      Properties.Settings.Default.MGMapsMode = (Properties.Settings.Default.OperatingMode > 2);

      ValidateChildren(ValidationConstraints.Enabled);

      if ((Properties.Settings.Default.OperatingMode == 1) || (Properties.Settings.Default.OperatingMode == 2))
      {
        switch (imageFormatComboBox.Text)
        {
          case "PNG":
            saveImageFileDialog.DefaultExt = "png";
            saveImageFileDialog.Filter = Properties.Resources.PngWildcard;
            break;
          case "JPEG":
            saveImageFileDialog.DefaultExt = "jpg";
            saveImageFileDialog.Filter = Properties.Resources.JpegWildcard;
            break;
          case "TIFF":
            saveImageFileDialog.DefaultExt = "tif";
            saveImageFileDialog.Filter = Properties.Resources.TiffWildcard;
            break;
        }

        saveImageFileDialog.FileName = "";
        saveImageFileDialog.ShowDialog();

        if (String.IsNullOrEmpty(saveImageFileDialog.FileName))
        {
          return;
        }
      }
      else if (Properties.Settings.Default.OperatingMode == 4)
      {
        openFileDialog.ShowDialog();

        if (String.IsNullOrEmpty(openFileDialog.FileName))
        {
          return;
        }
      }
      
      Cursor.Current = Cursors.WaitCursor;
      saveButton.Enabled = false;
      pw = null;

      if (Properties.Settings.Default.OperatingMode < 4)
      {
        if ((Properties.Settings.Default.TopLat < Properties.Settings.Default.BottomLat) || (Properties.Settings.Default.LeftLong > Properties.Settings.Default.RightLong))
        {
          ShowErrorBox(Properties.Resources.ExceptionLatLongIncorrect);
        }
        else if ((Math.Abs(Properties.Settings.Default.TopLat) > GMapTile.AbsLatMax) ||
                 (Math.Abs(Properties.Settings.Default.BottomLat) > GMapTile.AbsLatMax) ||
                 (Math.Abs(Properties.Settings.Default.LeftLong) > GMapTile.AbsLngMax) ||
                 (Math.Abs(Properties.Settings.Default.RightLong) > GMapTile.AbsLngMax))
        {
          ShowErrorBox(Properties.Resources.ExceptionLatLongOutside);
        }
        else
        {
          GMapTile TopLeft = GMapTile.GetTileFromLatLongZoom(Properties.Settings.Default.TopLat, Properties.Settings.Default.LeftLong, Properties.Settings.Default.ZoomLevel);
          GMapTile BottomRight = GMapTile.GetTileFromLatLongZoom(Properties.Settings.Default.BottomLat, Properties.Settings.Default.RightLong, Properties.Settings.Default.ZoomLevel);

          int wh = (int)Math.Sqrt(Properties.Settings.Default.NumberOfSlices);
          for (int i = 0; i < wh && !((pw != null) && pw.Cancelled); i++)
          {
            for (int j = 0; j < wh && !((pw != null) && pw.Cancelled); j++)
            {
              pw = new ProgressWindow(computeMapBackgroundWorker, String.Format(" - {0}/{1}", i * wh + j + 1, wh * wh));

              WorkerArguments arguments = new WorkerArguments();
              arguments.TopLeft = new GMapTile(TopLeft.X + ((BottomRight.X - TopLeft.X) * i) / wh + ((i == 0) ? 0 : Math.Sign(BottomRight.X - TopLeft.X)), TopLeft.Y + ((BottomRight.Y - TopLeft.Y) * j) / wh + ((j == 0) ? 0 : Math.Sign(BottomRight.Y - TopLeft.Y)), TopLeft.Zoom);
              arguments.BottomRight = new GMapTile(TopLeft.X + ((BottomRight.X - TopLeft.X) * (i + 1)) / wh, TopLeft.Y + ((BottomRight.Y - TopLeft.Y) * (j + 1)) / wh, BottomRight.Zoom);

              if (Properties.Settings.Default.OperatingMode != 0)
              {
                if (wh == 1)
                {
                  arguments.Filename = saveImageFileDialog.FileName;
                }
                else
                {
                  arguments.Filename = Path.Combine(Path.GetPathRoot(saveImageFileDialog.FileName), Path.GetDirectoryName(saveImageFileDialog.FileName));
                  arguments.Filename = Path.Combine(arguments.Filename, String.Format("{0}_{1}-{2}{3}", Path.GetFileNameWithoutExtension(saveImageFileDialog.FileName), i + 1, j + 1, Path.GetExtension(saveImageFileDialog.FileName)));
                }
              }

              computeMapBackgroundWorker.RunWorkerAsync(arguments);

              pw.ShowDialog();
            }
          }
        }
      }
      else
      {
        //MGMaps .map file mode
        uint f = 0;
        foreach (string fileName in openFileDialog.FileNames)
        {
          f++;

          if ((pw != null) && pw.Cancelled)
          {
            break;
          }

          string[] lines = File.ReadAllLines(fileName, Encoding.ASCII);

          switch (lines[0].Trim())
          {
            case "GoogleSat":
              Properties.Settings.Default.TileType = 0;
              break;
            case "GoogleMap":
              Properties.Settings.Default.TileType = 1;
              break;
            case "GoogleHyb":
              Properties.Settings.Default.TileType = 2;
              break;
            case "GoogleTer":
              Properties.Settings.Default.TileType = 3;
              break;
            default:
              throw new InvalidOperationException(String.Format(Properties.Resources.ExceptionBadMapFile, fileName));
          }
          lines[0] = null;

          uint l = 0;
          foreach (string line in lines)
          {
            l++;

            if (String.IsNullOrEmpty(line))
            {
              continue;
            }

            if ((pw != null) && pw.Cancelled)
            {
              break;
            }

            Match m = Regex.Match(line.Trim(), @"(?<z1>\d+)\s*-\s*(?<z2>\d+)\s*:\s*(?<lat1>-?[\d.]*\d+)\s*,\s*(?<lng1>-?[\d.]*\d+)\s*:\s*(?<lat2>-?[\d.]*\d+)\s*,\s*(?<lng2>-?[\d.]*\d+)", RegexOptions.CultureInvariant);
            uint zMin = uint.Parse(m.Groups["z1"].Value, ciUS.NumberFormat);
            uint zMax = uint.Parse(m.Groups["z2"].Value, ciUS.NumberFormat);
            double lat1 = double.Parse(m.Groups["lat1"].Value, ciUS.NumberFormat);
            double lat2 = double.Parse(m.Groups["lat2"].Value, ciUS.NumberFormat);
            double lng1 = double.Parse(m.Groups["lng1"].Value, ciUS.NumberFormat);
            double lng2 = double.Parse(m.Groups["lng2"].Value, ciUS.NumberFormat);

            if ((lat2 < lat1) || (lng1 > lng2))
            {
              ShowErrorBox(Properties.Resources.ExceptionLatLongIncorrect);
              break;
            }

            if ((Math.Abs(lat1) > GMapTile.AbsLatMax) ||
                (Math.Abs(lat2) > GMapTile.AbsLatMax) ||
                (Math.Abs(lng1) > GMapTile.AbsLngMax) ||
                (Math.Abs(lng2) > GMapTile.AbsLngMax))
            {
              ShowErrorBox(Properties.Resources.ExceptionLatLongOutside);
              break;
            }

            for (uint z = zMin; z <= zMax; z++)
            {
              if ((pw != null) && pw.Cancelled)
              {
                break;
              }

              pw = new ProgressWindow(computeMapBackgroundWorker, String.Format(" - File {0}, Zoom {1}, Line {2}/{3}", f, z, l - 1 , lines.Length - 1));

              WorkerArguments arguments = new WorkerArguments();
              arguments.TopLeft = GMapTile.GetTileFromLatLongZoom(lat2, lng1, z);
              arguments.BottomRight = GMapTile.GetTileFromLatLongZoom(lat1, lng2, z);
              arguments.Filename = null;

              computeMapBackgroundWorker.RunWorkerAsync(arguments);

              pw.ShowDialog();
            }
          }
        }
      }

      Cursor.Current = Cursors.Default;
      saveButton.Enabled = true;
    }

    private void tlLatTextBox_Validated(object sender, EventArgs e)
    {
      errorProvider.SetError(tlLatTextBox, "");
    }

    private void tlLatTextBox_Validating(object sender, CancelEventArgs e)
    {
      double d;
      if (!GMapTile.ParseLat(tlLatTextBox.Text, out d))
      {
        e.Cancel = true;
        tlLatTextBox.Select(0, tlLatTextBox.Text.Length);
        errorProvider.SetError(tlLatTextBox, resources.GetString("tlLatTextBox.Error"));
      }
      else
      {
        Properties.Settings.Default.TopLat = d;
        updateStatus(sender, e);
      }
    }

    private void tlLongTextBox_Validated(object sender, EventArgs e)
    {
      errorProvider.SetError(tlLongTextBox, "");
    }

    private void tlLongTextBox_Validating(object sender, CancelEventArgs e)
    {
      double d;
      if (!GMapTile.ParseLong(tlLongTextBox.Text, out d))
      {
        e.Cancel = true;
        tlLongTextBox.Select(0, tlLongTextBox.Text.Length);
        errorProvider.SetError(tlLongTextBox, resources.GetString("tlLatTextBox.Error"));
      }
      else
      {
        Properties.Settings.Default.LeftLong = d;
        updateStatus(sender, e);
      }
    }

    private void brLatTextBox_Validated(object sender, EventArgs e)
    {
      errorProvider.SetError(brLatTextBox, "");
    }

    private void brLatTextBox_Validating(object sender, CancelEventArgs e)
    {
      double d;
      if (!GMapTile.ParseLat(brLatTextBox.Text, out d))
      {
        e.Cancel = true;
        brLatTextBox.Select(0, brLatTextBox.Text.Length);
        errorProvider.SetError(brLatTextBox, resources.GetString("tlLatTextBox.Error"));
      }
      else
      {
        Properties.Settings.Default.BottomLat = d;
        updateStatus(sender, e);
      }
    }

    private void brLongTextBox_Validated(object sender, EventArgs e)
    {
      errorProvider.SetError(brLongTextBox, "");
    }

    private void brLongTextBox_Validating(object sender, CancelEventArgs e)
    {
      double d;
      if (!GMapTile.ParseLong(brLongTextBox.Text, out d))
      {
        e.Cancel = true;
        brLongTextBox.Select(0, brLongTextBox.Text.Length);
        errorProvider.SetError(brLongTextBox, resources.GetString("tlLatTextBox.Error"));
      }
      else
      {
        Properties.Settings.Default.RightLong = d;
        updateStatus(sender, e);
      }
    }

    private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
    {
      Properties.Settings.Default.Save();
    }

    private void googleMVtextBox_Validated(object sender, EventArgs e)
    {
      Properties.Settings.Default.CacheFullPath = googleMVtextBox.Text;
    }

    private void zoomLevelComboBox_Validated(object sender, EventArgs e)
    {
      Properties.Settings.Default.ZoomLevel = Convert.ToUInt32(zoomLevelComboBox.SelectedItem);
    }

    private void grayscalCheckBox_Validated(object sender, EventArgs e)
    {
      Properties.Settings.Default.GrayScale = grayscaleCheckBox.Checked;
      grayscalCheckBox_CheckedChanged(sender, e);
    }

    private void imageFormatComboBox_Validated(object sender, EventArgs e)
    {
      Properties.Settings.Default.ImageFormat = imageFormatComboBox.Text;
    }

    private void tileTypeComboBox_Validated(object sender, EventArgs e)
    {
      Properties.Settings.Default.TileType = tileTypeComboBox.SelectedIndex;
      tileTypeComboBox_SelectedIndexChanged(sender, e);
    }

    private void ditherCheckBox_Validated(object sender, EventArgs e)
    {
      Properties.Settings.Default.Dither = ditherCheckBox.Checked;
      ditherCheckBox_CheckedChanged(sender, e);
    }

    private void ditherCheckBox_CheckedChanged(object sender, EventArgs e)
    {
      if (ditherCheckBox.Checked)
      {
        grayscaleCheckBox.Checked = false;
      }
      updateStatus(sender, e);
    }

    private void grayscalCheckBox_CheckedChanged(object sender, EventArgs e)
    {
      if (grayscaleCheckBox.Checked)
      {
        ditherCheckBox.Checked = false;
      }
      updateStatus(sender, e);
    }

    private void operatingModeComboBox_Validated(object sender, EventArgs e)
    {
      Properties.Settings.Default.OperatingMode = operatingModeComboBox.SelectedIndex;
      operatingModeComboBox_SelectedIndexChanged(sender, e);
    }

    private void operatingModeComboBox_SelectedIndexChanged(object sender, EventArgs e)
    {
      switch (operatingModeComboBox.SelectedIndex)
      {
        default:
        case 1: //download & build image
        case 2: //download, build image & build .map
          imageGroupBox.Enabled = true;
          slicingComboBox.Enabled = true;
          ggZoneGroupBox.Enabled = true;
          statusLabel.Enabled = true;
          break;
        case 0: //download only
        case 3: //MGMaps mode, download only
          slicingComboBox.Enabled = false;
          imageGroupBox.Enabled = false;
          statusLabel.Enabled = false;
          ggZoneGroupBox.Enabled = true;
          break;
        case 4: //MGMaps mode, download from .map
          slicingComboBox.Enabled = false;
          imageGroupBox.Enabled = false;
          ggZoneGroupBox.Enabled = false;
          statusLabel.Enabled = false;
          break;
      }
    }

    private void aboutButton_Click(object sender, EventArgs e)
    {
      AboutBox ab = new AboutBox();
      ab.ShowDialog();
    }

    private void updateStatus(object sender, EventArgs e)
    {
      if ((Properties.Settings.Default.BottomLat > Properties.Settings.Default.TopLat) || (Properties.Settings.Default.RightLong < Properties.Settings.Default.LeftLong)
          || (slicingComboBox.SelectedIndex < 0) || (Properties.Settings.Default.OperatingMode == 0) || (Properties.Settings.Default.OperatingMode > 2))
      {
        statusLabel.Text = String.Format(resources.GetString("statusLabel.Text"), "??", "??", "??", "??", "");
      }
      else
      {
        uint zoom = uint.Parse(zoomLevelComboBox.Text);
        GMapTile TopLeft = GMapTile.GetTileFromLatLongZoom(Properties.Settings.Default.TopLat, Properties.Settings.Default.LeftLong, zoom);
        GMapTile BottomRight = GMapTile.GetTileFromLatLongZoom(Properties.Settings.Default.BottomLat, Properties.Settings.Default.RightLong, zoom);

        int nSlice = (int)((slicingComboBox.SelectedIndex + 1) * (slicingComboBox.SelectedIndex + 1));
        long width = (Math.Abs(TopLeft.X - BottomRight.X) + 1) * GMapTile.TILE_SIZE;
        long height = (Math.Abs(TopLeft.Y - BottomRight.Y) + 1) * GMapTile.TILE_SIZE;
        long memorySizePixel = width * height / nSlice;
        long memoryNeeded = 24 * 1024 * 1024 + memorySizePixel * (grayscaleCheckBox.Checked ? 1 : 3) * (ditherCheckBox.Checked ? 2 : 1);

        statusLabel.Text = String.Format(resources.GetString("statusLabel.Text"), width, height, memorySizePixel / 1000000, memoryNeeded / (1024 * 1024), (nSlice != 1) ? String.Format(" X{0}", nSlice) : "");
      }
    }

    private void proxyUseCheckBox_Validated(object sender, EventArgs e)
    {
      Properties.Settings.Default.UseProxyForSatelliteDownload = proxyUseCheckBox.Checked;
      proxyUseCheckBox_CheckedChanged(sender, e);
    }

    private void proxyUseCheckBox_CheckedChanged(object sender, EventArgs e)
    {
      if (proxyUseCheckBox.Checked)
      {
        waitRequestsCheckBox.Checked = true;
        delayNumericUpDown.Value = 150;
        waitRequestsCheckBox.Enabled = false;
        proxyListUrlsTextBox.Enabled = true;
        proxyListRegexpTextBox.Enabled = true;
      }
      else
      {
        waitRequestsCheckBox.Enabled = true;
        proxyListUrlsTextBox.Enabled = false;
        proxyListRegexpTextBox.Enabled = false;
      }
      waitRequestsCheckBox_CheckedChanged(sender, e);
    }

    private void waitRequestsCheckBox_Validated(object sender, EventArgs e)
    {
      Properties.Settings.Default.SlowDown = waitRequestsCheckBox.Checked;
      waitRequestsCheckBox_CheckedChanged(sender, e);
    }

    private void waitRequestsCheckBox_CheckedChanged(object sender, EventArgs e)
    {
      delayNumericUpDown.Enabled = waitRequestsCheckBox.Checked && waitRequestsCheckBox.Enabled;
      delayLabel.Enabled = waitRequestsCheckBox.Checked && waitRequestsCheckBox.Enabled;
    }

    private void delayNumericUpDown_Validated(object sender, EventArgs e)
    {
      Properties.Settings.Default.SlowDownDelay = (uint)delayNumericUpDown.Value;
    }

    private void proxyListUrlsTextBox_Validated(object sender, EventArgs e)
    {
      Properties.Settings.Default.ProxyListURLs = proxyListUrlsTextBox.Text;
      //errorProvider.SetError(proxyListUrlsTextBox, "");
    }

    private void proxyListUrlsTextBox_Validating(object sender, CancelEventArgs e)
    {
      /*try
      {
        TileDownloader.GetProxListFromURLs(proxyListUrlsTextBox.Text, null);
      }
      catch (Exception ex)
      {
        e.Cancel = true;
        proxyListUrlsTextBox.Select(0, proxyListUrlsTextBox.Text.Length);
        errorProvider.SetError(proxyListUrlsTextBox, resources.GetString("proxyListUrlsTextBox.Error") + "\n" + ex.Message);
      }*/
    }

    private void transpComboBox_Validated(object sender, EventArgs e)
    {
      Properties.Settings.Default.DrivingMapTransparency = Convert.ToUInt32(transpComboBox.SelectedItem.ToString().TrimEnd(new char[] { '%' }));
    }

    private void tileTypeComboBox_SelectedIndexChanged(object sender, EventArgs e)
    {
      if (tileTypeComboBox.SelectedIndex != 2)
      {
        transpComboBox.Enabled = false;
        transpLabel.Enabled = false;
      }
      else
      {
        transpComboBox.Enabled = true;
        transpLabel.Enabled = true;
      }
    }

    private void slicingComboBox_Validated(object sender, EventArgs e)
    {
      Properties.Settings.Default.NumberOfSlices = (uint)((slicingComboBox.SelectedIndex + 1) * (slicingComboBox.SelectedIndex + 1));
    }

    private void proxyListRegexpTextBox_Validated(object sender, EventArgs e)
    {
      Properties.Settings.Default.ProxyListRegexp = proxyListRegexpTextBox.Text;
    }

    private void resetButton_Click(object sender, EventArgs e)
    {
      Properties.Settings.Default.Reset();
      Properties.Settings.Default.Save();
      InitUIwithValues();
    }
  }
}